@@ -1,39 +1,35 @@
 #pragma once
 
 #include <string>
+#include <format>
+#include "SharedValues.hpp"
 
 inline static constexpr auto ROUNDED_SHADER_FUNC = [](const std::string colorVarName) -> std::string {
     return R"#(
 
-    // branchless baby!
+    // shoutout me: I fixed this shader being a bit pixelated while watching hentai
+
     highp vec2 pixCoord = vec2(gl_FragCoord);
     pixCoord -= topLeft + fullSize * 0.5;
     pixCoord *= vec2(lessThan(pixCoord, vec2(0.0))) * -2.0 + 1.0;
     pixCoord -= fullSize * 0.5 - radius;
+    pixCoord += vec2(1.0, 1.0) / fullSize; // center the pix dont make it top-left
+
+    // smoothing constant for the edge: more = blurrier, but smoother
+    const float SMOOTHING_CONSTANT = )#" +
+        std::format("{:.7f}", SHADER_ROUNDED_SMOOTHING_FACTOR) + R"#(;
 
     if (pixCoord.x + pixCoord.y > radius) {
 
-	    float dist = length(pixCoord);
+	      float dist = pow(pow(pixCoord.x, roundingPower) + pow(pixCoord.y, roundingPower), 1.0/roundingPower);
 
-	    if (dist > radius)
-	        discard;
+	      if (dist > radius + SMOOTHING_CONSTANT)
+	          discard;
 
-	    if (primitiveMultisample == 1 && dist > radius - 1.0) {
-	        float distances = 0.0;
-	        distances += float(length(pixCoord + vec2(0.25, 0.25)) < radius);
-	        distances += float(length(pixCoord + vec2(0.75, 0.25)) < radius);
-	        distances += float(length(pixCoord + vec2(0.25, 0.75)) < radius);
-	        distances += float(length(pixCoord + vec2(0.75, 0.75)) < radius);
-
-	        if (distances == 0.0)
-		        discard;
-
-	        distances /= 4.0;
-
-	        )#" +
-        colorVarName + R"#( = )#" + colorVarName + R"#( * distances;
-        }
+        float normalized = 1.0 - smoothstep(0.0, 1.0, (dist - radius + SMOOTHING_CONSTANT) / (SMOOTHING_CONSTANT * 2.0));
 
+	      )#" +
+        colorVarName + R"#( = )#" + colorVarName + R"#( * normalized;
     }
 )#";
 };
@@ -43,24 +39,26 @@ uniform mat3 proj;
 uniform vec4 color;
 attribute vec2 pos;
 attribute vec2 texcoord;
+attribute vec2 texcoordMatte;
 varying vec4 v_color;
 varying vec2 v_texcoord;
+varying vec2 v_texcoordMatte;
 
 void main() {
     gl_Position = vec4(proj * vec3(pos, 1.0), 1.0);
     v_color = color;
     v_texcoord = texcoord;
+    v_texcoordMatte = texcoordMatte;
 })#";
 
 inline const std::string QUADFRAGSRC = R"#(
-precision mediump float;
+precision highp float;
 varying vec4 v_color;
 
 uniform vec2 topLeft;
 uniform vec2 fullSize;
 uniform float radius;
-
-uniform int primitiveMultisample;
+uniform float roundingPower;
 
 void main() {
 
@@ -85,8 +83,19 @@ void main() {
     v_texcoord = texcoord;
 })#";
 
+inline const std::string TEXVERTSRC320 = R"#(#version 320 es
+uniform mat3 proj;
+in vec2 pos;
+in vec2 texcoord;
+out vec2 v_texcoord;
+
+void main() {
+    gl_Position = vec4(proj * vec3(pos, 1.0), 1.0);
+    v_texcoord = texcoord;
+})#";
+
 inline const std::string TEXFRAGSRCRGBA = R"#(
-precision mediump float;
+precision highp float;
 varying vec2 v_texcoord; // is in 0-1
 uniform sampler2D tex;
 uniform float alpha;
@@ -94,6 +103,7 @@ uniform float alpha;
 uniform vec2 topLeft;
 uniform vec2 fullSize;
 uniform float radius;
+uniform float roundingPower;
 
 uniform int discardOpaque;
 uniform int discardAlpha;
@@ -102,15 +112,13 @@ uniform float discardAlphaValue;
 uniform int applyTint;
 uniform vec3 tint;
 
-uniform int primitiveMultisample;
-
 void main() {
 
     vec4 pixColor = texture2D(tex, v_texcoord);
 
     if (discardOpaque == 1 && pixColor[3] * alpha == 1.0)
 	    discard;
-    
+
     if (discardAlpha == 1 && pixColor[3] <= discardAlphaValue)
         discard;
 
@@ -129,7 +137,7 @@ void main() {
 })#";
 
 inline const std::string TEXFRAGSRCRGBAPASSTHRU = R"#(
-precision mediump float;
+precision highp float;
 varying vec2 v_texcoord; // is in 0-1
 uniform sampler2D tex;
 
@@ -137,8 +145,18 @@ void main() {
     gl_FragColor = texture2D(tex, v_texcoord);
 })#";
 
+inline const std::string TEXFRAGSRCRGBAMATTE = R"#(
+precision highp float;
+varying vec2 v_texcoord; // is in 0-1
+uniform sampler2D tex;
+uniform sampler2D texMatte;
+
+void main() {
+    gl_FragColor = texture2D(tex, v_texcoord) * texture2D(texMatte, v_texcoord)[0]; // I know it only uses R, but matte should be black/white anyways.
+})#";
+
 inline const std::string TEXFRAGSRCRGBX = R"#(
-precision mediump float;
+precision highp float;
 varying vec2 v_texcoord;
 uniform sampler2D tex;
 uniform float alpha;
@@ -146,6 +164,7 @@ uniform float alpha;
 uniform vec2 topLeft;
 uniform vec2 fullSize;
 uniform float radius;
+uniform float roundingPower;
 
 uniform int discardOpaque;
 uniform int discardAlpha;
@@ -154,8 +173,6 @@ uniform int discardAlphaValue;
 uniform int applyTint;
 uniform vec3 tint;
 
-uniform int primitiveMultisample;
-
 void main() {
 
     if (discardOpaque == 1 && alpha == 1.0)
@@ -179,12 +196,113 @@ void main() {
 
 inline const std::string FRAGBLUR1 = R"#(
 #version 100
-precision mediump float;
-varying mediump vec2 v_texcoord; // is in 0-1
-uniform sampler2D tex;
+precision            highp float;
+varying highp vec2   v_texcoord; // is in 0-1
+uniform sampler2D    tex;
 
-uniform float radius;
-uniform vec2 halfpixel;
+uniform float        radius;
+uniform vec2         halfpixel;
+uniform int          passes;
+uniform float        vibrancy;
+uniform float        vibrancy_darkness;
+
+// see http://alienryderflex.com/hsp.html
+const float Pr = 0.299;
+const float Pg = 0.587;
+const float Pb = 0.114;
+
+// Y is "v" ( brightness ). X is "s" ( saturation )
+// see https://www.desmos.com/3d/a88652b9a4
+// Determines if high brightness or high saturation is more important
+const float a = 0.93;
+const float b = 0.11;
+const float c = 0.66; //  Determines the smoothness of the transition of unboosted to boosted colors
+//
+
+// http://www.flong.com/archive/texts/code/shapers_circ/
+float doubleCircleSigmoid(float x, float a) {
+    a = clamp(a, 0.0, 1.0);
+
+    float y = .0;
+    if (x <= a) {
+        y = a - sqrt(a * a - x * x);
+    } else {
+        y = a + sqrt(pow(1. - a, 2.) - pow(x - 1., 2.));
+    }
+    return y;
+}
+
+vec3 rgb2hsl(vec3 col) {
+    float red   = col.r;
+    float green = col.g;
+    float blue  = col.b;
+
+    float minc  = min(col.r, min(col.g, col.b));
+    float maxc  = max(col.r, max(col.g, col.b));
+    float delta = maxc - minc;
+
+    float lum = (minc + maxc) * 0.5;
+    float sat = 0.0;
+    float hue = 0.0;
+
+    if (lum > 0.0 && lum < 1.0) {
+        float mul = (lum < 0.5) ? (lum) : (1.0 - lum);
+        sat       = delta / (mul * 2.0);
+    }
+
+    if (delta > 0.0) {
+        vec3  maxcVec = vec3(maxc);
+        vec3  masks = vec3(equal(maxcVec, col)) * vec3(notEqual(maxcVec, vec3(green, blue, red)));
+        vec3  adds = vec3(0.0, 2.0, 4.0) + vec3(green - blue, blue - red, red - green) / delta;
+
+        hue += dot(adds, masks);
+        hue /= 6.0;
+
+        if (hue < 0.0)
+            hue += 1.0;
+    }
+
+    return vec3(hue, sat, lum);
+}
+
+vec3 hsl2rgb(vec3 col) {
+    const float onethird = 1.0 / 3.0;
+    const float twothird = 2.0 / 3.0;
+    const float rcpsixth = 6.0;
+
+    float       hue = col.x;
+    float       sat = col.y;
+    float       lum = col.z;
+
+    vec3        xt = vec3(0.0);
+
+    if (hue < onethird) {
+        xt.r = rcpsixth * (onethird - hue);
+        xt.g = rcpsixth * hue;
+        xt.b = 0.0;
+    } else if (hue < twothird) {
+        xt.r = 0.0;
+        xt.g = rcpsixth * (twothird - hue);
+        xt.b = rcpsixth * (hue - onethird);
+    } else
+        xt = vec3(rcpsixth * (hue - twothird), 0.0, rcpsixth * (1.0 - hue));
+
+    xt = min(xt, 1.0);
+
+    float sat2   = 2.0 * sat;
+    float satinv = 1.0 - sat;
+    float luminv = 1.0 - lum;
+    float lum2m1 = (2.0 * lum) - 1.0;
+    vec3  ct     = (sat2 * xt) + satinv;
+
+    vec3  rgb;
+    if (lum >= 0.5)
+        rgb = (luminv * ct) + lum2m1;
+    else
+        rgb = lum * ct;
+
+    return rgb;
+}
 
 void main() {
     vec2 uv = v_texcoord * 2.0;
@@ -195,14 +313,35 @@ void main() {
     sum += texture2D(tex, uv + vec2(halfpixel.x, -halfpixel.y) * radius);
     sum += texture2D(tex, uv - vec2(halfpixel.x, -halfpixel.y) * radius);
 
-    gl_FragColor = sum / 8.0;
+    vec4 color = sum / 8.0;
+
+    if (vibrancy == 0.0) {
+        gl_FragColor = color;
+    } else {
+        // Invert it so that it correctly maps to the config setting
+        float vibrancy_darkness1 = 1.0 - vibrancy_darkness;
+
+        // Decrease the RGB components based on their perceived brightness, to prevent visually dark colors from overblowing the rest.
+        vec3 hsl = rgb2hsl(color.rgb);
+        // Calculate perceived brightness, as not boost visually dark colors like deep blue as much as equally saturated yellow
+        float perceivedBrightness = doubleCircleSigmoid(sqrt(color.r * color.r * Pr + color.g * color.g * Pg + color.b * color.b * Pb), 0.8 * vibrancy_darkness1);
+
+        float b1        = b * vibrancy_darkness1;
+        float boostBase = hsl[1] > 0.0 ? smoothstep(b1 - c * 0.5, b1 + c * 0.5, 1.0 - (pow(1.0 - hsl[1] * cos(a), 2.0) + pow(1.0 - perceivedBrightness * sin(a), 2.0))) : 0.0;
+
+        float saturation = clamp(hsl[1] + (boostBase * vibrancy) / float(passes), 0.0, 1.0);
+
+        vec3  newColor = hsl2rgb(vec3(hsl[0], saturation, hsl[2]));
+
+        gl_FragColor = vec4(newColor, color[3]);
+    }
 }
 )#";
 
 inline const std::string FRAGBLUR2 = R"#(
 #version 100
-precision mediump float;
-varying mediump vec2 v_texcoord; // is in 0-1
+precision highp float;
+varying highp vec2 v_texcoord; // is in 0-1
 uniform sampler2D tex;
 
 uniform float radius;
@@ -225,40 +364,73 @@ void main() {
 }
 )#";
 
-inline const std::string FRAGBLURFINISH = R"#(
-precision mediump float;
-varying vec2 v_texcoord; // is in 0-1
+inline const std::string FRAGBLURPREPARE = R"#(
+precision         highp float;
+varying vec2      v_texcoord; // is in 0-1
 uniform sampler2D tex;
 
-uniform float contrast;
-uniform float noise;
-uniform float brightness;
+uniform float     contrast;
+uniform float     brightness;
 
-float hash(vec2 p) {
-	return fract(sin(dot(p, vec2(12.9898, 78.233))) * 43758.5453);
+float gain(float x, float k) {
+    float a = 0.5 * pow(2.0 * ((x < 0.5) ? x : 1.0 - x), k);
+    return (x < 0.5) ? a : 1.0 - a;
 }
 
 void main() {
     vec4 pixColor = texture2D(tex, v_texcoord);
 
     // contrast
-    pixColor.rgb = (pixColor.rgb - 0.5) * contrast + 0.5;
+    if (contrast != 1.0) {
+        pixColor.r = gain(pixColor.r, contrast);
+        pixColor.g = gain(pixColor.g, contrast);
+        pixColor.b = gain(pixColor.b, contrast);
+    }
 
     // brightness
-    pixColor.rgb *= brightness;
+    if (brightness > 1.0) {
+        pixColor.rgb *= brightness;
+    }
+
+    gl_FragColor = pixColor;
+}
+)#";
+
+inline const std::string FRAGBLURFINISH = R"#(
+precision         highp float;
+varying vec2      v_texcoord; // is in 0-1
+uniform sampler2D tex;
+
+uniform float     noise;
+uniform float     brightness;
+
+float hash(vec2 p) {
+    vec3 p3 = fract(vec3(p.xyx) * 1689.1984);
+    p3 += dot(p3, p3.yzx + 33.33);
+    return fract((p3.x + p3.y) * p3.z);
+}
+
+void main() {
+    vec4 pixColor = texture2D(tex, v_texcoord);
 
     // noise
-    float noiseHash = hash(v_texcoord);
+    float noiseHash   = hash(v_texcoord);
     float noiseAmount = (mod(noiseHash, 1.0) - 0.5);
     pixColor.rgb += noiseAmount * noise;
 
+    // brightness
+    if (brightness < 1.0) {
+        pixColor.rgb *= brightness;
+    }
+
     gl_FragColor = pixColor;
-})#";
+}
+)#";
 
 inline const std::string TEXFRAGSRCEXT = R"#(
 #extension GL_OES_EGL_image_external : require
 
-precision mediump float;
+precision highp float;
 varying vec2 v_texcoord;
 uniform samplerExternalOES texture0;
 uniform float alpha;
@@ -266,6 +438,7 @@ uniform float alpha;
 uniform vec2 topLeft;
 uniform vec2 fullSize;
 uniform float radius;
+uniform float roundingPower;
 
 uniform int discardOpaque;
 uniform int discardAlpha;
@@ -274,8 +447,6 @@ uniform int discardAlphaValue;
 uniform int applyTint;
 uniform vec3 tint;
 
-uniform int primitiveMultisample;
-
 void main() {
 
     vec4 pixColor = texture2D(texture0, v_texcoord);
@@ -299,7 +470,7 @@ void main() {
 )#";
 
 static const std::string FRAGGLITCH = R"#(
-precision mediump float;
+precision highp float;
 varying vec2 v_texcoord;
 uniform sampler2D tex;
 uniform float time; // quirk: time is set to 0 at the beginning, should be around 10 when crash.
@@ -363,4 +534,4 @@ void main() {
 
     gl_FragColor = pixColor;
 }
-)#";
\ No newline at end of file
+)#";
